/*
 VARNA is a tool for the automated drawing, visualization and annotation of the secondary structure of RNA, designed as a companion software for web servers and databases.
 Copyright (C) 2008  Kevin Darty, Alain Denise and Yann Ponty.
 electronic mail : Yann.Ponty@lri.fr
 paper mail : LRI, bat 490 Universit Paris-Sud 91405 Orsay Cedex France

 This file is part of VARNA version 3.1.
 VARNA version 3.1 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 VARNA version 3.1 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with VARNA version 3.1.
 If not, see http://www.gnu.org/licenses.
 */
package fr.orsay.lri.varna.models.rna;


import java.awt.Color;
import java.awt.Point;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Stack;
import java.util.Vector;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.InputSource;

import fr.orsay.lri.varna.exceptions.ExceptionExportFailed;
import fr.orsay.lri.varna.exceptions.ExceptionFileFormatOrSyntax;
import fr.orsay.lri.varna.exceptions.ExceptionLoadingFailed;
import fr.orsay.lri.varna.exceptions.ExceptionNAViewAlgorithm;
import fr.orsay.lri.varna.exceptions.ExceptionPermissionDenied;
import fr.orsay.lri.varna.exceptions.ExceptionUnmatchedClosingParentheses;
import fr.orsay.lri.varna.exceptions.ExceptionWritingForbidden;
import fr.orsay.lri.varna.interfaces.InterfaceVARNAListener;
import fr.orsay.lri.varna.interfaces.InterfaceVARNAObservable;
import fr.orsay.lri.varna.models.TextAnnotation;
import fr.orsay.lri.varna.models.VARNAConfig;
import fr.orsay.lri.varna.models.export.PSExport;
import fr.orsay.lri.varna.models.export.SVGExport;
import fr.orsay.lri.varna.models.export.SecStrDrawingProducer;
import fr.orsay.lri.varna.models.export.XFIGExport;
import fr.orsay.lri.varna.models.naView.NAView;

/**
 * The RNA model which contain the base list and the draw algorithm mode
 * 
 * @author darty
 * 
 */
public class RNA extends InterfaceVARNAObservable {
	/**
	 * Selects the "Feynman diagram" drawing algorithm that places the bases on
	 * a circle and draws the base-pairings as chords of the circle graph.
	 */
	public static final int DRAW_MODE_CIRCULAR = 1;
	/**
	 * Selects the "tree drawing" algorithm. Draws each loop on a circle whose
	 * radius depends on the number of bases involved in the loop. As some
	 * helices can be overlapping in the result, basic interaction is provided
	 * so that the user can "disentangle" the drawing by spinning the helices
	 * around the axis defined by their multiloop (bulge or internal loop)
	 * origin. This is roughly the initial placement strategy of RNAViz.
	 * 
	 * @see <a href="http://rnaviz.sourceforge.net/">RNAViz</a>
	 */
	public static final int DRAW_MODE_RADIATE = 2;

	/**
	 * Selects the NAView algorithm.
	 */
	public static final int DRAW_MODE_NAVIEW = 3;

	/**
	 * Selects the line algorithm.
	 */
	public static final int DRAW_MODE_LINEAR = 4;

	private Double _spaceBetweenBases = 1.0;
	/**
	 * The draw algorithm mode
	 */
	private int _drawMode = DRAW_MODE_RADIATE;

	public static int BASE_RADIUS = 10;
	public static double LOOP_DISTANCE = 40.0;
	public static double BASE_PAIR_DISTANCE = 65.0;
	public static double MULTILOOP_DISTANCE = 35.0;
	public static double BP_HEIGHT_INCREMENT = 0.65;
	
	
	public static final int FILE_TYPE_BPSEQ = 1;
	public static final int FILE_TYPE_CT = 2;
	public static final int FILE_TYPE_DBN = 3;
	public static final int FILE_TYPE_RNAML = 4;
	public static final int FILE_TYPE_UNKNOWN = 5;

	private boolean _comparisonMode;
	private boolean _drawn = false;
	/**
	 * the base list
	 */
	private ArrayList<ModeleBase> _listeBases;
	/**
	 * Additional bonds and info can be specified here.
	 */
	private ArrayList<ModeleStyleBP> _structureAux = new ArrayList<ModeleStyleBP>();

	private ArrayList<InterfaceVARNAListener> _listeVARNAListener = new ArrayList<InterfaceVARNAListener>();

	ArrayList<Character> _normalBases = new ArrayList<Character>();
	
	private ArrayList<TextAnnotation> _listeAnnotations = new ArrayList<TextAnnotation>();

	boolean _flatExteriorLoop = false;
	
	private String _name = "";

	public RNA() {
		this("");
	}
	public RNA(String n) {
		_name = n;
		_comparisonMode = false;
		_listeBases = new ArrayList<ModeleBase>();
		_drawn = false;
		init();
	}

	public String toString()
	{
		if (_name.equals(""))
		{
			return getStructDBN();
		}
		else
		{
			return _name; 
		}
	}
	
	public RNA(RNA r) {
		_spaceBetweenBases = r._spaceBetweenBases;
		_drawMode = r._drawMode;
		_comparisonMode = r._comparisonMode;
		_listeBases.addAll(r._listeBases);
		_listeVARNAListener = (ArrayList<InterfaceVARNAListener>) r._listeVARNAListener;
		_drawn = r._drawn;
		init();
	}

	public void init() {
		_normalBases.add('a');
		_normalBases.add('c');
		_normalBases.add('g');
		_normalBases.add('u');
		_normalBases.add('-');
	}

	public void saveRNADBN(String path, String title)
			throws ExceptionWritingForbidden {
		try {
			FileWriter out = new FileWriter(path);
			if (!title.equals("")) {
				out.write("> " + title + "\n");
			}
			out.write(getListeBasesToString());
			out.write('\n');
			String str = "";
			for (int i = 0; i < _listeBases.size(); i++) {
				if (_listeBases.get(i).get_elementStructure() == -1) {
					str += '.';
				} else {
					if (_listeBases.get(i).get_elementStructure() > i) {
						str += '(';
					} else {
						str += ')';
					}
				}
			}
			out.write(str);
			out.write('\n');
			out.close();
		} catch (IOException e) {
			throw new ExceptionWritingForbidden(e.getMessage());
		}
	}

	public Color getBaseInnerColor(int i, VARNAConfig conf) {
		Color result = _listeBases.get(i).get_styleBase()
				.get_base_inner_color();
		char res = _listeBases.get(i).getContent().charAt(0);
		if ((conf._colorSpecialBases && !_normalBases.contains(Character
				.toLowerCase(res)))) {
			result = conf._specialBasesColor;
		} else if ((conf._colorDashBases && (Character.toLowerCase(res) == '-'))) {
			result = conf._dashBasesColor;
		}
		return result;
	}

	public Color getBaseOuterColor(int i, VARNAConfig conf) {
		Color result = _listeBases.get(i).get_styleBase()
				.get_base_outline_color();
		return result;
	}

	public Color getBaseNameColor(int i, VARNAConfig conf) {
		Color result = _listeBases.get(i).get_styleBase().get_base_name_color();
		return result;
	}

	public Color getBasePairColor(ModeleStyleBP bp, VARNAConfig conf) {
		Color bondColor = conf._bondColor;
		if (conf._useBaseColorsForBPs) {
			bondColor = _listeBases.get(bp.getPartner5().getIndex())
					.get_styleBase().get_base_inner_color();
		}
		if (bp != null) {
			bondColor = bp.getColor(bondColor);
		}
		return bondColor;
	}

	public double getBasePairThickness(ModeleStyleBP bp, VARNAConfig conf) {
		double thickness = bp.getThickness(conf._bpThickness);
		return thickness;
	}

	private void drawSymbol(SecStrDrawingProducer out, double posx,
			double posy, double normx, double normy, double radius,
			boolean isCIS, ModeleStyleBP.Edge e, double thickness) {
		Color bck = out.getCurrentColor();
		switch (e) {
		case WATSON_CRICK:
			if (isCIS) {
				out.fillCircle(posx, posy, (radius / 2.0), thickness, bck);
			} else {
				out.drawCircle(posx, posy, (radius / 2.0), thickness);
				out.fillCircle(posx, posy, (radius / 2.0), thickness,
						Color.white);
			}
			break;
		case HOOGSTEEN: {
			double xtab[] = new double[4];
			double ytab[] = new double[4];
			xtab[0] = posx - radius * normx / 2.0 - radius * normy / 2.0;
			ytab[0] = posy - radius * normy / 2.0 + radius * normx / 2.0;
			xtab[1] = posx + radius * normx / 2.0 - radius * normy / 2.0;
			ytab[1] = posy + radius * normy / 2.0 + radius * normx / 2.0;
			xtab[2] = posx + radius * normx / 2.0 + radius * normy / 2.0;
			ytab[2] = posy + radius * normy / 2.0 - radius * normx / 2.0;
			xtab[3] = posx - radius * normx / 2.0 + radius * normy / 2.0;
			ytab[3] = posy - radius * normy / 2.0 - radius * normx / 2.0;
			if (isCIS) {
				out.fillPolygon(xtab, ytab, thickness, bck);
			} else {
				out.drawPolygon(xtab, ytab, thickness);
				out.fillPolygon(xtab, ytab, thickness, Color.white);
			}
		}
			break;
		case SUGAR: {
			double ix = radius * normx / 2.0;
			double iy = radius * normy / 2.0;
			double jx = radius * normy / 2.0;
			double jy = -radius * normx / 2.0;
			double xtab[] = new double[3];
			double ytab[] = new double[3];
			xtab[0] = posx - ix + jx;
			ytab[0] = posy - iy + jy;
			xtab[1] = posx + ix + jx;
			ytab[1] = posy + iy + jy;
			xtab[2] = posx - jx;
			ytab[2] = posy - jy;

			if (isCIS) {
				out.fillPolygon(xtab, ytab, thickness, bck);
			} else {
				out.drawPolygon(xtab, ytab, thickness);
				out.fillPolygon(xtab, ytab, thickness, Color.white);
			}
		}
			break;
		}
	}

	private void drawBasePair(SecStrDrawingProducer out, Point2D.Double orig,
			Point2D.Double dest, ModeleStyleBP style, VARNAConfig conf) {
		if (conf._mainBPStyle == VARNAConfig.BP_STYLE.BP_STYLE_LW) {
			double thickness = getBasePairThickness(style, conf);
			double radiusCircle = ((RNA.BASE_PAIR_DISTANCE - RNA.BASE_RADIUS) / 5.0);
			double dx = dest.x - orig.x;
			double dy = dest.y - orig.y;
			double dist = Math.sqrt((dest.x - orig.x) * (dest.x - orig.x)
					+ (dest.y - orig.y) * (dest.y - orig.y));
			dx /= dist;
			dy /= dist;
			double nx = -dy;
			double ny = dx;
			if (style.isCanonical()) {
				if (style.isCanonicalGC()) {
					if ((orig.x != dest.x) || (orig.y != dest.y)) {
						nx *= RNA.BASE_RADIUS / 4.0;
						ny *= RNA.BASE_RADIUS / 4.0;
						out
								.drawLine((orig.x + nx), (orig.y + ny),
										(dest.x + nx), (dest.y + ny),
										conf._bpThickness);
						out
								.drawLine((orig.x - nx), (orig.y - ny),
										(dest.x - nx), (dest.y - ny),
										conf._bpThickness);
					}
				} else if (style.isWobbleUG()) {
					double cx = (dest.x + orig.x) / 2.0;
					double cy = (dest.y + orig.y) / 2.0;
					out.drawLine(orig.x, orig.y, dest.x, dest.y,
							conf._bpThickness);
					drawSymbol(out, cx, cy, nx, ny, radiusCircle,
							style.isCIS(), style.getEdgePartner5(), thickness);
				} else {
					out.drawLine(orig.x, orig.y, dest.x, dest.y,
							conf._bpThickness);
				}
			} else {
				ModeleStyleBP.Edge p1 = style.getEdgePartner5();
				ModeleStyleBP.Edge p2 = style.getEdgePartner3();
				double cx = (dest.x + orig.x) / 2.0;
				double cy = (dest.y + orig.y) / 2.0;
				out.drawLine(orig.x, orig.y, dest.x, dest.y, conf._bpThickness);
				if (p1 == p2) {
					drawSymbol(out, cx, cy, nx, ny, radiusCircle,
							style.isCIS(), p1, thickness);
				} else {
					double vdx = (dest.x - orig.x);
					double vdy = (dest.y - orig.y);
					vdx /= 8.0;
					vdy /= 8.0;
					drawSymbol(out, cx + vdx, cy + vdy, nx, ny, radiusCircle,
							style.isCIS(), p1, thickness);
					drawSymbol(out, cx - vdx, cy - vdy, nx, ny, radiusCircle,
							style.isCIS(), p2, thickness);
				}
			}
		} else if (conf._mainBPStyle == VARNAConfig.BP_STYLE.BP_STYLE_RNAVIZ) {
			double xcenter = (orig.x + dest.x) / 2.0;
			double ycenter = (orig.y + dest.y) / 2.0;
			out.fillCircle(xcenter, ycenter, 3.0 * conf._bpThickness,
					conf._bpThickness, out.getCurrentColor());
		} else if (conf._mainBPStyle == VARNAConfig.BP_STYLE.BP_STYLE_SIMPLE) {
			out.drawLine(orig.x, orig.y, dest.x, dest.y, conf._bpThickness);
		}
	}

	private void saveRNA(String path, VARNAConfig conf, double scale,
			SecStrDrawingProducer out) throws ExceptionWritingForbidden {
		out.setScale(scale);

		// Computing bounding boxes
		double EPSMargin = 40;
		double minX = Double.MAX_VALUE;
		double maxX = Double.MIN_VALUE;
		double minY = Double.MAX_VALUE;
		double maxY = Double.MIN_VALUE;

		double x0, y0, x1, y1, dx, dy, norm;

		for (int i = 0; i < _listeBases.size(); i++) {
			minX = Math.min(minX, (_listeBases.get(i).get_coords().getX()
					- BASE_RADIUS - EPSMargin));
			minY = Math.min(minY, -(_listeBases.get(i).get_coords().getY()
					- BASE_RADIUS - EPSMargin));
			maxX = Math.max(maxX, (_listeBases.get(i).get_coords().getX()
					+ BASE_RADIUS + EPSMargin));
			maxY = Math.max(maxY, -(_listeBases.get(i).get_coords().getY()
					+ BASE_RADIUS + EPSMargin));
		}

		// Drawing backbone
		out.setColor(conf._backboneColor);
		for (int i = 1; i < _listeBases.size(); i++) {
			x0 = (_listeBases.get(i - 1).get_coords().getX() - minX);
			y0 = -(_listeBases.get(i - 1).get_coords().getY() - minY);
			x1 = (_listeBases.get(i).get_coords().getX() - minX);
			y1 = -(_listeBases.get(i).get_coords().getY() - minY);
			out.drawLine(x0, y0, x1, y1, 1);
		}

		// Drawing bonds
		for (int i = 0; i < _listeBases.size(); i++) {
			if (_listeBases.get(i).get_elementStructure() > i) {
				ModeleStyleBP style = _listeBases.get(i).get_styleBP();
				if (style.isCanonical() || conf._drawnNonCanonicalBP) {
					Color bpcol = getBasePairColor(style, conf);
					out.setColor(bpcol);

					int j = _listeBases.get(i).get_elementStructure();
					x0 = (_listeBases.get(i).get_coords().getX() - minX);
					y0 = -(_listeBases.get(i).get_coords().getY() - minY);
					x1 = (_listeBases.get(j).get_coords().getX() - minX);
					y1 = -(_listeBases.get(j).get_coords().getY() - minY);
					dx = x1 - x0;
					dy = y1 - y0;
					norm = Math.sqrt(dx * dx + dy * dy);
					dx /= norm;
					dy /= norm;

					if (_drawMode == DRAW_MODE_CIRCULAR
							|| _drawMode == DRAW_MODE_RADIATE
							|| _drawMode == DRAW_MODE_NAVIEW) {
						drawBasePair(out, new Point2D.Double(x0, y0),
								new Point2D.Double(x1, y1), style, conf);
					} else if (_drawMode == DRAW_MODE_LINEAR) {
						double coef;
						double distance;
						if (j - i == 1)
							coef = BP_HEIGHT_INCREMENT * 2;
						else
							coef = BP_HEIGHT_INCREMENT * 1;
						distance = (int) Math.round(x1 - x0);
						out.drawArc(new Point2D.Double(x0, y0), distance,
								distance * coef, 0, 180);
					}
				}
			}
		}

		// Drawing additional bonds
		if (conf._drawnNonPlanarBP) {
			for (int i = 0; i < _structureAux.size(); i++) {
				ModeleStyleBP bp = _structureAux.get(i);
				out.setColor(getBasePairColor(bp, conf));

				int a = bp.getPartner5().getIndex();
				int b = bp.getPartner3().getIndex();

				if (bp.isCanonical() || conf._drawnNonCanonicalBP) {
					x0 = (_listeBases.get(a).get_coords().getX() - minX);
					y0 = -(_listeBases.get(a).get_coords().getY() - minY);
					x1 = (_listeBases.get(b).get_coords().getX() - minX);
					y1 = -(_listeBases.get(b).get_coords().getY() - minY);
					dx = x1 - x0;
					dy = y1 - y0;
					norm = Math.sqrt(dx * dx + dy * dy);
					dx /= norm;
					dy /= norm;
					if ((_drawMode == DRAW_MODE_CIRCULAR)
							|| (_drawMode == DRAW_MODE_RADIATE)
							|| _drawMode == DRAW_MODE_NAVIEW) {
						drawBasePair(out, new Point2D.Double(x0, y0),
								new Point2D.Double(x1, y1), bp, conf);
					} else if (_drawMode == DRAW_MODE_LINEAR) {
						double coef;
						double distance;
						if (b - a == 1)
							coef = BP_HEIGHT_INCREMENT * 2;
						else
							coef = BP_HEIGHT_INCREMENT * 1;
						distance = (int) Math.round(x1 - x0);
						out.drawArc(new Point2D.Double(x0, y0), distance,
								distance * coef, 0, 180);
					}
				}
			}
		}

		// Drawing Bases
		double baseFontSize = (1.5 * BASE_RADIUS);
		out.setFont(PSExport.FONT_HELVETICA_BOLD, baseFontSize);
		if (_comparisonMode) {
			for (int i = 0; i < _listeBases.size(); i++) {
				x0 = (_listeBases.get(i).get_coords().getX() - minX);
				y0 = -(_listeBases.get(i).get_coords().getY() - minY);
				out.fillCircle(x0, y0, BASE_RADIUS, 1l, getBaseInnerColor(i,
						conf));
				out.setColor(getBaseOuterColor(i, conf));
				out.drawCircle(x0, y0, BASE_RADIUS, 1l);
				out.setColor(getBaseNameColor(i, conf));
				out.drawText(x0, y0, ((ModeleBasesComparison) _listeBases
						.get(i)).getBases());
			}
		} else {
			for (int i = 0; i < _listeBases.size(); i++) {
				ModeleBase mb = _listeBases.get(i);
				x0 = (mb.get_coords().getX() - minX);
				y0 = -(mb.get_coords().getY() - minY);
				out.fillCircle(x0, y0, BASE_RADIUS, 1l, getBaseInnerColor(i,
						conf));
				out.setColor(getBaseOuterColor(i, conf));
				out.drawCircle(x0, y0, BASE_RADIUS, 1l);
				out.setColor(getBaseNameColor(i, conf));
				out.drawText(x0, y0, _listeBases.get(i).getContent());
			}
		}

		// Drawing base numbers
		double numFontSize = (double) (1.5 * BASE_RADIUS);
		out.setFont(PSExport.FONT_HELVETICA_BOLD, numFontSize);

		for (int i = 0; i < _listeBases.size(); i++) {
			int basenum = _listeBases.get(i).getBaseNumber();
			if (basenum == -1) {
				basenum = i + 1;
			}
			if (this.isNumberDrawn(_listeBases.get(i),conf._numPeriod)) {
				x0 = (_listeBases.get(i).get_coords().getX() - minX);
				y0 = -(_listeBases.get(i).get_coords().getY() - minY);
				x1 = (_listeBases.get(i).get_center().getX() - minX);
				y1 = -(_listeBases.get(i).get_center().getY() - minY);
				dx = x1 - x0;
				dy = y1 - y0;
				norm = Math.sqrt(dx * dx + dy * dy);
				dx /= norm;
				dy /= norm;
				out.drawLine((x0 - 1.5 * BASE_RADIUS * dx), (y0 - 1.5
						* BASE_RADIUS * dy), (x0 - 2.5 * BASE_RADIUS * dx),
						(y0 - 2.5 * BASE_RADIUS * dy), 1);
				out.drawText((x0 - (conf._distNumbers+1.0) * BASE_RADIUS * dx), (y0 - (conf._distNumbers+1.0) * BASE_RADIUS
						* dy), "" + (basenum));
			}
		}
		renderAnnotations(out, minX, minY,conf);
		
		// Drawing Title
		Rectangle2D.Double currentBBox = out.getBoundingBox(); 
		double titleFontSize = (2.0*conf._titleFont.getSize());
		out.setColor(conf._titleColor);
		out.setFont(PSExport.FONT_HELVETICA_BOLD,titleFontSize);
		double yTitle = currentBBox.y-titleFontSize/2.0;
		if (!getName().equals(""))
		{
			out.drawText((maxX-minX)/2.0,yTitle, getName());			
		}
		else if (!conf._title.equals(""))
		{
			out.drawText((maxX-minX)/2.0,yTitle, getName());			
		}
			
		

		FileWriter fout;
		try {
			fout = new FileWriter(path);
			fout.write(out.export());
			fout.close();
		} catch (IOException e) {
			throw new ExceptionWritingForbidden(e.getMessage());
		}
	}
	
	Point2D.Double buildCaptionPosition(ModeleBase mb, double heightEstimate, VARNAConfig conf)
	{
		double radius = 2.0;
		if (isNumberDrawn(mb,conf._numPeriod))
		{ radius += (conf._distNumbers+1.0); }
		Point2D.Double center = mb.get_center();
		Point2D.Double p = mb.get_coords();
		double realDistance = RNA.BASE_RADIUS*radius + heightEstimate;
		return new Point2D.Double(center.getX()
		+ (p.getX() - center.getX())
		* ((p.distance(center) + realDistance) / p
				.distance(center)), center.getY()
		+ (p.getY() - center.getY())
		* ((p.distance(center) + realDistance) / p
				.distance(center)) );
	}

	
	private void renderAnnotations(SecStrDrawingProducer out, double minX, double minY, VARNAConfig conf) 
	{
		for (TextAnnotation textAnnotation : getAnnotations()) {
			out.setColor(textAnnotation.getColor());
			out.setFont(PSExport.FONT_HELVETICA_BOLD,2.0*textAnnotation.getFont().getSize());
			Point2D.Double position = textAnnotation.getCenterPosition();
			if (textAnnotation.getType()==TextAnnotation.BASE)
			{
				ModeleBase mb = (ModeleBase) textAnnotation.getAncrage();
				double fontHeight = Math.ceil(textAnnotation.getFont().getSize());
				position = buildCaptionPosition(mb,fontHeight,conf);

			}
			out.drawText(position.x-minX,-(position.y-minY), textAnnotation.getTexte());
		}
	}


	public boolean isNumberDrawn(ModeleBase mb, int numPeriod)
	{
		return (mb.getIndex() == 0) || ((mb.getBaseNumber()) % numPeriod == 0)
		|| (mb.getIndex() == get_listeBases().size() - 1);
	}
	
	
	public void saveRNAEPS(String path, VARNAConfig conf)
			throws ExceptionWritingForbidden {
		PSExport out = new PSExport();
		saveRNA(path, conf, 0.4, out);
	}

	public void saveRNAXFIG(String path, VARNAConfig conf)
			throws ExceptionWritingForbidden {
		XFIGExport out = new XFIGExport();
		saveRNA(path, conf, 20, out);
	}

	public void saveRNASVG(String path, VARNAConfig conf)
			throws ExceptionWritingForbidden {
		SVGExport out = new SVGExport();
		saveRNA(path, conf, 0.5, out);
	}

	public Rectangle2D.Double getBBox() {
		Rectangle2D.Double result = new Rectangle2D.Double(11, 10, 10, 10);

		double minx, maxx, miny, maxy;
		minx = Double.MAX_VALUE;
		miny = Double.MAX_VALUE;
		maxx = -Double.MAX_VALUE;
		maxy = -Double.MAX_VALUE;
		for (int i = 0; i < _listeBases.size(); i++) {
			minx = Math.min(_listeBases.get(i).get_coords().getX()
					- BASE_RADIUS, minx);
			miny = Math.min(_listeBases.get(i).get_coords().getY()
					- BASE_RADIUS, miny);
			maxx = Math.max(_listeBases.get(i).get_coords().getX()
					+ BASE_RADIUS, maxx);
			maxy = Math.max(_listeBases.get(i).get_coords().getY()
					+ BASE_RADIUS, maxy);
		}
		result.x = minx;
		result.y = miny;
		result.width = (maxx - minx);
		result.height = (maxy - miny);
		return result;
	}

	public void setCoord(int index, Point2D.Double p) {
		setCoord(index, p.x, p.y);
	}

	public void setCoord(int index, double x, double y) {
		if (index < _listeBases.size()) {
			_listeBases.get(index).set_coords(new Point2D.Double(x, y));
		}
	}

	public Point2D.Double getCoords(int i) {
		if (i < _listeBases.size() && i >= 0) {
			return _listeBases.get(i).get_coords();
		}
		return new Point2D.Double();
	}

	public String getBaseContent(int i) {
		if ((i>=0)&&(i < _listeBases.size())) 
		{ return _listeBases.get(i).getContent(); }
		return "";
	}

	public int getBaseNumber(int i) {
		if ((i>=0)&&(i < _listeBases.size())) 
		{ return _listeBases.get(i).getBaseNumber(); }
		return -1;
	}
	
	public Point2D.Double getCenter(int i) {
		if (i < _listeBases.size()) {
			return _listeBases.get(i).get_center();
		}

		return new Point2D.Double();
	}

	public void setCenter(int i, Point2D.Double p) {
		if (i < _listeBases.size()) {
			_listeBases.get(i).set_center(p);
		}
	}

	public void drawRNACircle() {
		_drawn = true;
		_drawMode = DRAW_MODE_CIRCULAR;
		int radius = (int) ((3 * (_listeBases.size() + 1) * BASE_RADIUS) / (2 * Math.PI));
		double angle;
		for (int i = 0; i < _listeBases.size(); i++) {
			angle = -((((double) -(i + 1)) * 2.0 * Math.PI)
					/ ((double) (_listeBases.size() + 1)) - Math.PI / 2.0);
			_listeBases.get(i).set_coords(
					new Point2D.Double(
							(radius * Math.cos(angle) * _spaceBetweenBases),
							(radius * Math.sin(angle) * _spaceBetweenBases)));
			_listeBases.get(i).set_center(new Point2D.Double(0, 0));
		}
	}

	public void drawRNALine() {
		_drawn = true;
		_drawMode = DRAW_MODE_LINEAR;
		for (int i = 0; i < get_listeBases().size(); i++) {
			get_listeBases().get(i).set_coords(
					new Point2D.Double(i * _spaceBetweenBases * 20, 0));
			get_listeBases().get(i).set_center(
					new Point2D.Double(i * _spaceBetweenBases * 20, -10));
		}
	}

	private double objFun(int n1, int n2, double r) {
		return (((double) n1) * 2.0
				* Math.asin(((double) BASE_PAIR_DISTANCE) / (2.0 * r))
				+ ((double) n2) * 2.0
				* Math.asin(((double) MULTILOOP_DISTANCE) / (2.0 * r)) - (2.0 * Math.PI));
	}

	private double determineRadius(int n1, int n2, double startRadius) {
		@SuppressWarnings("unused")
		double increment = 0.5;
		double xmin = BASE_PAIR_DISTANCE / 2.0;
		double xmax = 3.0 * BASE_PAIR_DISTANCE + 1;
		double x = (xmin + xmax) / 2.0;
		double y = 10000.0;
		double ymin = -1000.0;
		double ymax = 1000.0;
		int numIt = 0;
		double precision = 0.00001;
		while ((Math.abs(y) > precision) && (numIt < 10000)) {
			x = (xmin + xmax) / 2.0;
			y = objFun(n1, n2, x);
			ymin = objFun(n1, n2, xmax);
			ymax = objFun(n1, n2, xmin);
			if (ymin > 0.0) {
				xmax = xmax + (xmax - xmin);
			} else if ((y <= 0.0) && (ymax > 0.0)) {
				xmax = x;
			} else if ((y >= 0.0) && (ymin < 0.0)) {
				xmin = x;
			} else if (ymax < 0.0) {
				xmin = Math.max(xmin - (x - xmin), Math.max(
						BASE_PAIR_DISTANCE / 2.0, MULTILOOP_DISTANCE / 2.0));
				xmax = x;
			}
			numIt++;
		}
		// if (Math.abs(y)>precision)
		// System.out.println("x: ["+xmin+","+x+","+xmax+"] inc:"+increment+" y: ["+ymin+","+y+","+ymax+"]");
		return x;
	}

	private void drawLoop(int i, int j, double x, double y, double dirAngle,
			Point2D.Double[] coords, Point2D.Double[] centers) {
		if (i > j) {
			return;
		}

		// BasePaired
		if (_listeBases.get(i).get_elementStructure() == j) {
			double normalAngle = Math.PI / 2.0;
			centers[i] = new Point2D.Double(x, y);
			centers[j] = new Point2D.Double(x, y);
			coords[i].x = (x + BASE_PAIR_DISTANCE
					* Math.cos(dirAngle - normalAngle) / 2.0);
			coords[i].y = (y + BASE_PAIR_DISTANCE
					* Math.sin(dirAngle - normalAngle) / 2.0);
			coords[j].x = (x + BASE_PAIR_DISTANCE
					* Math.cos(dirAngle + normalAngle) / 2.0);
			coords[j].y = (y + BASE_PAIR_DISTANCE
					* Math.sin(dirAngle + normalAngle) / 2.0);
			drawLoop(i + 1, j - 1, x + LOOP_DISTANCE * Math.cos(dirAngle), y
					+ LOOP_DISTANCE * Math.sin(dirAngle), dirAngle, coords,
					centers);
		} else {
			int k = i;
			Vector<Integer> basesMultiLoop = new Vector<Integer>();
			Vector<Integer> helices = new Vector<Integer>();
			int l;
			while (k <= j) {
				l = _listeBases.get(k).get_elementStructure();
				if (l > k) {
					basesMultiLoop.add(new Integer(k));
					basesMultiLoop.add(new Integer(l));
					helices.add(new Integer(k));
					k = l + 1;
				} else {
					basesMultiLoop.add(new Integer(k));
					k++;
				}
			}
			int mlSize = basesMultiLoop.size() + 2;
			int numHelices = helices.size() + 1;
			double totalLength = MULTILOOP_DISTANCE * (mlSize - numHelices)
					+ BASE_PAIR_DISTANCE * numHelices;
			double multiLoopRadius;
			double angleIncrementML;
			double angleIncrementBP;
			if (mlSize > 3) {
				multiLoopRadius = determineRadius(numHelices, mlSize
						- numHelices, (totalLength) / (2.0 * Math.PI));
				angleIncrementML = -2.0
						* Math.asin(((float) MULTILOOP_DISTANCE)
								/ (2.0 * multiLoopRadius));
				angleIncrementBP = -2.0
						* Math.asin(((float) BASE_PAIR_DISTANCE)
								/ (2.0 * multiLoopRadius));
			} else {
				multiLoopRadius = 35.0;
				angleIncrementBP = -2.0
						* Math.asin(((float) BASE_PAIR_DISTANCE)
								/ (2.0 * multiLoopRadius));
				angleIncrementML = (-2.0 * Math.PI - angleIncrementBP) / 2.0;
			}
			// System.out.println("MLr:"+multiLoopRadius+" iBP:"+angleIncrementBP+" iML:"+angleIncrementML);

			double centerDist = Math.sqrt(Math.max(Math.pow(multiLoopRadius, 2)
					- Math.pow(BASE_PAIR_DISTANCE / 2.0, 2), 0.0))
					- LOOP_DISTANCE;
			Point2D.Double mlCenter = new Point2D.Double(
					(x + (centerDist * Math.cos(dirAngle))),
					(y + (centerDist * Math.sin(dirAngle))));

			// Base directing angle for (multi|hairpin) loop, from the center's
			// perspective
			double baseAngle = dirAngle
			// U-turn
					+ Math.PI
					// Account for already drawn supporting base-pair
					+ 0.5 * angleIncrementBP
					// Base cannot be paired twice, so next base is at
					// "unpaired base distance"
					+ 1.0 * angleIncrementML;
			double[] angles = new double[_listeBases.size()];
			int n1 = 1;
			int n2 = 1;
			for (k = basesMultiLoop.size() - 1; k >= 0; k--) {
				l = basesMultiLoop.get(k).intValue();
				centers[l] = mlCenter;
				angles[l] = baseAngle;
				coords[l].x = mlCenter.x + multiLoopRadius
						* Math.cos(baseAngle);
				coords[l].y = mlCenter.y + multiLoopRadius
						* Math.sin(baseAngle);
				if ((_listeBases.get(l).get_elementStructure() < l)
						&& (_listeBases.get(l).get_elementStructure() != -1)) {
					baseAngle += angleIncrementBP;
					n1++;
				} else {
					baseAngle += angleIncrementML;
					n2++;
				}
			}
			// System.out.println("n1:"+n1+" n2:"+n2);
			double newAngle;
			int m, n;
			for (k = 0; k < helices.size(); k++) {
				m = helices.get(k).intValue();
				n = _listeBases.get(m).get_elementStructure();
				newAngle = (angles[m] + angles[n]) / 2.0;
				drawLoop(m + 1, n - 1, (LOOP_DISTANCE * Math.cos(newAngle))
						+ (coords[m].x + coords[n].x) / 2.0,
						(LOOP_DISTANCE * Math.sin(newAngle))
								+ (coords[m].y + coords[n].y) / 2.0, newAngle,
						coords, centers);
			}
		}
	}

	public void drawRNA(int mode) throws ExceptionNAViewAlgorithm {
		_drawMode = mode;
		switch (get_drawMode()) {
		case RNA.DRAW_MODE_RADIATE:
			drawRNARadiate();
			break;
		case RNA.DRAW_MODE_LINEAR:
			drawRNALine();
			break;
		case RNA.DRAW_MODE_CIRCULAR:
			drawRNACircle();
			break;
		case RNA.DRAW_MODE_NAVIEW:
			drawRNANAView();
			break;
		default:
			break;
		}

	}

	public int getDrawMode() {
		return _drawMode;
	}


	public void drawRNARadiate() {
		drawRNARadiate(-1.0,_flatExteriorLoop);
	}
	
	public void drawRNARadiate(double dirAngle, boolean flatExteriorLoop) {
		_drawn = true;
		_drawMode = DRAW_MODE_RADIATE;
		Point2D.Double[] coords = new Point2D.Double[_listeBases.size()];
		Point2D.Double[] centers = new Point2D.Double[_listeBases.size()];
		for (int i = 0; i < _listeBases.size(); i++) {
			coords[i] = new Point2D.Double(0, 0);
			centers[i] = new Point2D.Double(0, 0);
		}
		if (flatExteriorLoop)
		{
		  dirAngle += 1.0 - Math.PI/2.0;
		  int i=0;
		  double x = 0.0;
		  double y = 0.0;
		  double vx = -Math.sin(dirAngle);
		  double vy = Math.cos(dirAngle);
		  while(i<_listeBases.size())
		  {
			  coords[i].x = x; 
			  coords[i].y = y;
			  centers[i].x = x+BASE_PAIR_DISTANCE*vy;
			  centers[i].y = y-BASE_PAIR_DISTANCE*vx;
			  int j = _listeBases.get(i).get_elementStructure();
			  if (j>i)
			  {
				  drawLoop(i, j, x+(BASE_PAIR_DISTANCE*vx/2.0), y+(BASE_PAIR_DISTANCE*vy/2.0), dirAngle, coords, centers);
				  centers[i].x = coords[i].x+BASE_PAIR_DISTANCE*vy;
				  centers[i].y = y-BASE_PAIR_DISTANCE*vx;
				  i = j;
				  x += BASE_PAIR_DISTANCE*vx;
				  y += BASE_PAIR_DISTANCE*vy;
				  centers[i].x = coords[i].x+BASE_PAIR_DISTANCE*vy;
				  centers[i].y = y-BASE_PAIR_DISTANCE*vx;
			  }
			  x += MULTILOOP_DISTANCE*vx;
			  y += MULTILOOP_DISTANCE*vy;
			  i += 1;
		  }
		}
		else
		{
		  drawLoop(0, _listeBases.size() - 1, 0, 0, dirAngle, coords, centers);
		}
		for (int i = 0; i < _listeBases.size(); i++) {
			_listeBases.get(i).set_coords(
					new Point2D.Double(coords[i].x * _spaceBetweenBases,
							coords[i].y * _spaceBetweenBases));
			_listeBases.get(i).set_center(
					new Point2D.Double(centers[i].x * _spaceBetweenBases,
							centers[i].y * _spaceBetweenBases));
		}

		// TODO
		// change les centres des bases de la premiere helice vers la boucle la
		// plus proche
	}

	public void drawRNANAView() throws ExceptionNAViewAlgorithm {
		_drawMode = DRAW_MODE_NAVIEW;
		_drawn = true;

		ArrayList<Double> X = new ArrayList<Double>(_listeBases.size());
		ArrayList<Double> Y = new ArrayList<Double>(_listeBases.size());
		ArrayList<Short> pair_table = new ArrayList<Short>(_listeBases.size());

		for (int i = 0; i < _listeBases.size(); i++) {
			pair_table.add(Short.valueOf(String.valueOf(_listeBases.get(i)
					.get_elementStructure())));
		}
		NAView naView = new NAView();
		naView.naview_xy_coordinates(pair_table, X, Y);

		// Updating individual base positions
		for (int i = 0; i < _listeBases.size(); i++) {
			_listeBases.get(i).set_coords(
					new Point2D.Double(X.get(i) * 2.5 * _spaceBetweenBases, Y
							.get(i)
							* 2.5 * _spaceBetweenBases));
		}

		// Updating centers
		for (int i = 0; i < _listeBases.size(); i++) {
			int indicePartner = _listeBases.get(i).get_elementStructure();
			if (indicePartner != -1) {
				Point2D.Double base = _listeBases.get(i).get_coords();
				Point2D.Double partner = _listeBases.get(indicePartner)
						.get_coords();
				_listeBases.get(i).set_center(
						new Point2D.Double((base.x + partner.x) / 2.0,
								(base.y + partner.y) / 2.0));
			} else {
				Vector<Integer> loop = getLoopBases(i);
				double tmpx = 0.0;
				double tmpy = 0.0;
				for (int j = 0; j < loop.size(); j++) {
					int partner = loop.elementAt(j);
					Point2D.Double loopmember = _listeBases.get(partner)
							.get_coords();
					tmpx += loopmember.x;
					tmpy += loopmember.y;
				}
				_listeBases.get(i).set_center(
						new Point2D.Double(tmpx / loop.size(), tmpy
								/ loop.size()));
			}
		}
	}

	public int get_drawMode() {
		return _drawMode;
	}

	public void set_drawMode(int drawMode) {
		_drawMode = drawMode;
	}

	public void setRNA(char[] seq, int[] str)
		throws ExceptionFileFormatOrSyntax 
	{
		setRNA(seq, str, 1);
	}
	
	public void setRNA(char[] seq, int[] str, int baseIndex)
			throws ExceptionFileFormatOrSyntax {
		clearAnnotations();
		_listeBases = new ArrayList<ModeleBase>();
		if (seq.length != str.length) {
			warningEmition("Sequence length " + seq.length
					+ " differs from that of secondary structure " + str.length
					+ ". \nAdapting sequence length ...");
			if (seq.length < str.length) {
				String seqtmp = String.copyValueOf(seq);
				while (seqtmp.length() < str.length) {
					seqtmp += " ";
				}
				seq = seqtmp.toCharArray();
			} else {
				seq = String.copyValueOf(seq).substring(0, str.length)
						.toCharArray();
			}
		}
		for (int i = 0; i < str.length; i++) {
			_listeBases.add(new ModeleBaseNucleotide(seq[i], i, baseIndex+i));
		}
		applyStruct(str);
	}

	/**
	 * Sets the RNA to be drawn. Uses when comparison mode is on. Will draw the
	 * super-structure passed in parameters and apply specials styles to the
	 * bases owning by each RNA alignment and both.
	 * 
	 * @param seq
	 *            - The sequence of the super-structure This sequence shall be
	 *            designed like this:
	 *            <code>firstRNA1stBaseSecondRNA1stBaseFirstRNA2ndBaseSecondRNA2ndBase [...]</code>
	 * <br>
	 *            <b>Example:</b> <code>AAC-GUAGA--UGG</code>
	 * @param struct
	 *            - The super-structure
	 * @param basesOwn
	 *            - The RNA owning bases array (each index will be:0 when common
	 *            base, 1 when first RNA alignment base, 2 when second RNA
	 *            alignment base)
	 * @throws ExceptionUnmatchedClosingParentheses
	 * @throws ExceptionFileFormatOrSyntax 
	 */
	public void setRNA(String seq, String struct, ArrayList<Integer> basesOwn)
			throws ExceptionUnmatchedClosingParentheses, ExceptionFileFormatOrSyntax {
		clearAnnotations();
		_listeBases = new ArrayList<ModeleBase>();
		// On "parse" la structure (repérage des points, tiret et couples
		// parentheses ouvrante/fermante)
		int[] array_struct = parseStruct(struct);
		// On enregistre la taille de la structure pour éviter d'appeler la
		// fonction à chaque passage dans la boucle for ci-après
		int size = struct.length();
		// Compteur pour les caractères (on va aller de deux en deux)
		int j = 0;
		boolean simple = false;
		for (int i = 0; i < size; i++) {
			// On créer une nouvelle base de comparaison avec les deux bases
			// courantes
			// if (seq.charAt(j) != seq.charAt(j + 1)) {
			_listeBases.add(new ModeleBasesComparison(seq.charAt(j), seq
					.charAt(j + 1), i));
			// }
			// Si les deux bases sont les mêmes, alors on dessinne une base
			// simple
			// else {
			// _listeBases.add(new ModeleBaseNucleotide(seq.charAt(j)));
			// simple = true;
			// }
			// On applique la structure à cette base
			_listeBases.get(i).set_elementStructure(array_struct[i]);
			// Non-testé: On renseigne à quelle ARN cette base appartient à
			// l'origine (en termes de structure).
			if (!simple)
				((ModeleBasesComparison) _listeBases.get(i))
						.set_appartenance(basesOwn.get(i));
			// S'il s'agit d'une base simple
			// else {
			// System.out.println("This is "+_listeBases.get(i)+" for "+i);
			// if (basesOwn.get(i) == 0) {
			// ((ModeleBaseNucleotide)
			// _listeBases.get(i)).get_styleBase().set_base_inner_color(ModeleBasesComparison.BOTH_RNA_COLOR);
			// }
			// else if (basesOwn.get(i) == 1){
			// ((ModeleBaseNucleotide)
			// _listeBases.get(i)).get_styleBase().set_base_inner_color(ModeleBasesComparison.FIRST_RNA_COLOR);
			// }
			// else if ((basesOwn.get(i) == 2)) {
			// ((ModeleBaseNucleotide)
			// _listeBases.get(i)).get_styleBase().set_base_inner_color(ModeleBasesComparison.SECOND_RNA_COLOR);
			// }
			// }
			// On passe au morceau de la séquence suivante.
			j += 2;
		}
	}

	public void setRNA(String seq, int[] str)
			throws ExceptionFileFormatOrSyntax {
		setRNA(seq.toCharArray(), str);
	}

	public void setRNA(String seq, String dbnStr)
			throws ExceptionUnmatchedClosingParentheses,
			ExceptionFileFormatOrSyntax {
		clearAnnotations();
		String parDBN = dbnStr.replace('(', '(').replace(')', ')').replace('[', ':').replace(']', ':').replace('{', ':').replace('}', ':');
		String braDBN = dbnStr.replace('(', ':').replace(')', ':').replace('[', '(').replace(']', ')').replace('{', ':').replace('}', ':');
		String accDBN = dbnStr.replace('(', ':').replace(')', ':').replace('[', ':').replace(']', ':').replace('{', '(').replace('}', ')');
	    int[] parStr = parseStruct(parDBN);
		int[] braStr = parseStruct(braDBN);
		int[] accStr = parseStruct(accDBN);
		int[] finStr = new int[parStr.length];
		for(int i=0;i<parStr.length;i++)
		{ finStr[i] = -1; } 
		
		for(int i=0;i<parStr.length;i++)
		{
			if (parStr[i]>i)
			{
				finStr[i] = parStr[i];
				finStr[finStr[i]] = i;
			}
			else if (braStr[i]>i)
			{
				if ((parStr[i]==-1)&&(parStr[braStr[i]]==-1))
				{
					finStr[i] = braStr[i];
					finStr[finStr[i]] = i;				
				}
			}
			else if (accStr[i]>i)
			{
				if ((parStr[i]==-1)&&(parStr[accStr[i]]==-1)&&(braStr[i]==-1)&&(braStr[accStr[i]]==-1))
				{
					finStr[i] = accStr[i];
					finStr[finStr[i]] = i;				
				}
			}
		}
		setRNA(seq.toCharArray(), finStr);
	}

	public int[] parseStruct(String str)
			throws ExceptionUnmatchedClosingParentheses, ExceptionFileFormatOrSyntax {
		int[] result = new int[str.length()];
		int unexpectedChar = -1;
		Stack<Integer> p = new Stack<Integer>();
		for (int i = 0; i < str.length(); i++) {
			char c = str.charAt(i);
			if (c == '(') {
				p.push(new Integer(i));
			} else if (c == '.' || c == '-' || c == ':') {
				result[i] = -1;
			} else if (c == ')') {
				if (p.size() == 0) {
					throw new ExceptionUnmatchedClosingParentheses(i + 1);
				}
				int j = p.pop().intValue();
				result[i] = j;
				result[j] = i;
			} else {
				result[i] = -1;
				if (unexpectedChar == -1)
					unexpectedChar = i;
			}
		}

		if (unexpectedChar != -1) {
			//warningEmition("Unexpected Character at index:" + unexpectedChar);
			throw new ExceptionFileFormatOrSyntax("Unexpected Character at index:" + unexpectedChar); 
		}

		if (p.size() != 0) {
			throw new ExceptionUnmatchedClosingParentheses(
					p.pop().intValue() + 1);
		}

		return result;
	}

	public Point getHelix(int index) {
		if ((index < 0) || (index >= _listeBases.size())) {
			return new Point(index, index);
		}
		int j = _listeBases.get(index).get_elementStructure();
		if (j != -1) {
			int minH = index;
			int maxH = index;
			if (j > index) {
				maxH = j;
			} else {
				minH = j;
			}
			boolean over = false;
			while (!over) {
				if ((minH < 0) || (maxH >= _listeBases.size())) {
					over = true;
				} else {
					if (_listeBases.get(minH).get_elementStructure() == maxH) {
						minH--;
						maxH++;
					} else {
						over = true;
					}
				}
			}
			minH++;
			maxH--;
			return new Point(minH, maxH);
		}
		return new Point(0, 0);
	}

	public Point getMultiLoop(int index) {
		if ((index < 0) || (index >= _listeBases.size())) {
			return new Point(index, index);
		}
		Point h = getHelix(index);
		int minH = h.x - 1;
		int maxH = h.y + 1;
		boolean over = false;
		while (!over) {
			if (minH < 0) {
				over = true;
				minH = 0;
			} else {
				if (_listeBases.get(minH).get_elementStructure() == -1) {
					minH--;
				} else if (_listeBases.get(minH).get_elementStructure() < minH) {
					minH = _listeBases.get(minH).get_elementStructure() - 1;
				} else {
					over = true;
				}
			}
		}
		over = false;
		while (!over) {
			if (maxH > _listeBases.size() - 1) {
				over = true;
				maxH = _listeBases.size() - 1;
			} else {
				if (_listeBases.get(maxH).get_elementStructure() == -1) {
					maxH++;
				} else if (_listeBases.get(maxH).get_elementStructure() > maxH) {
					maxH = _listeBases.get(maxH).get_elementStructure() + 1;
				} else {
					over = true;
				}
			}
		}
		return new Point(minH, maxH);
	}

	public Vector<Integer> getLoopBases(int startIndex) {
		Vector<Integer> result = new Vector<Integer>();

		if ((startIndex < 0) || (startIndex >= _listeBases.size())) {
			return result;
		}
		int index = startIndex;
		result.add(startIndex);
		if (_listeBases.get(index).get_elementStructure() <= index) {
			index = (index + 1) % _listeBases.size();
		} else {
			index = _listeBases.get(index).get_elementStructure();
			result.add(index);
			index = (index + 1) % _listeBases.size();
		}

		while (index != startIndex) {
			result.add(index);
			if (_listeBases.get(index).get_elementStructure() == -1) {
				index = (index + 1) % _listeBases.size();
			} else {
				index = _listeBases.get(index).get_elementStructure();
				result.add(index);
				index = (index + 1) % _listeBases.size();
			}
		}
		return result;
	}

	/**
	 * Returns the RNA secondary structure displayed by this panel as a
	 * well-parenthesized word, accordingly to the DBN format
	 * 
	 * @return This panel's secondary structure
	 */
	public String getStructDBN() {
		String result = "";
		for (int i = 0; i < _listeBases.size(); i++) {
			int j = _listeBases.get(i).get_elementStructure();
			if (j == -1) {
				result += ".";
			} else if (i > j) {
				result += ")";
			} else {
				result += "(";
			}
		}
		return result;
	}

	public String getStructDBN(int[] str) {
		String result = "";
		for (int i = 0; i < str.length; i++) {
			if (str[i] == -1) {
				result += ".";
			} else if (str[i] > i) {
				result += "(";
			} else {
				result += ")";
			}
		}
		return result;
	}

	/**
	 * Returns the raw nucleotides sequence for the displayed RNA
	 * 
	 * @return The RNA sequence
	 */
	public String getSeq() {
		String result = "";
		if (_comparisonMode) {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += ((ModeleBasesComparison) _listeBases.get(i))
						.getBases();
			}
		} else {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += ((ModeleBaseNucleotide) _listeBases.get(i)).get_c();
			}
		}
		return result;
	}

	public String getStructBPSEQ() {
		String result = "";
		if (_comparisonMode) {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += (i + 1)
						+ " "
						+ ((ModeleBasesComparison) _listeBases.get(i))
								.getBases() + " "
						+ (_listeBases.get(i).get_elementStructure() + 1)
						+ "\n";
			}
		} else {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += (i + 1) + " "
						+ ((ModeleBaseNucleotide) _listeBases.get(i)).get_c()
						+ " " + (_listeBases.get(i).get_elementStructure() + 1)
						+ "\n";
			}
		}
		return result;
	}

	public String getStructCT() {
		String result = "";
		if (_comparisonMode) {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += (i + 1)
						+ " "
						+ ((ModeleBasesComparison) _listeBases.get(i))
								.getBases() + " " + i + " " + (i + 2) + " "
						+ (_listeBases.get(i).get_elementStructure() + 1) + " "
						+ (i + 1) + "\n";
			}
		} else {
			for (int i = 0; i < _listeBases.size(); i++) {
				result += (i + 1) + " "
						+ ((ModeleBaseNucleotide) _listeBases.get(i)).get_c()
						+ " " + i + " " + (i + 2) + " "
						+ (_listeBases.get(i).get_elementStructure() + 1) + " "
						+ (i + 1) + "\n";
			}
		}
		return result;
	}

	public void saveAsBPSEQ(String path, String title)
			throws ExceptionExportFailed, ExceptionPermissionDenied {
		try {
			FileWriter f = new FileWriter(path);
			f.write("# " + title + "\n");
			f.write(this.getStructBPSEQ() + "\n");
			f.close();
		} catch (IOException e) {
			throw new ExceptionExportFailed(e.getMessage(), path);
		}
	}

	public void saveAsCT(String path, String title)
			throws ExceptionExportFailed, ExceptionPermissionDenied {
		try {
			FileWriter f = new FileWriter(path);
			f.write("" + _listeBases.size() + " " + title + "\n");
			f.write(this.getStructCT() + "\n");
			f.close();
		} catch (IOException e) {
			throw new ExceptionExportFailed(e.getMessage(), path);
		}
	}

	public void saveAsDBN(String path, String title)
			throws ExceptionExportFailed, ExceptionPermissionDenied {
		try {
			FileWriter f = new FileWriter(path);
			f.write("> " + title + "\n");
			f.write(getListeBasesToString() + "\n");
			f.write(getStructDBN() + "\n");
			f.close();
		} catch (IOException e) {
			throw new ExceptionExportFailed(e.getMessage(), path);
		}
	}

	public String getListeBasesToString() {
		String s = new String();
		if (_comparisonMode) {
			for (int i = 0; i < _listeBases.size(); i++) {
				s += ((ModeleBasesComparison) _listeBases.get(i)).getBases();
			}
		} else {
			for (int i = 0; i < _listeBases.size(); i++) {
				s += ((ModeleBaseNucleotide) _listeBases.get(i)).get_c();
			}
		}
		return s;
	}

	private boolean loadSecStrBPSEQ(Reader r) throws ExceptionPermissionDenied,
			ExceptionLoadingFailed, ExceptionFileFormatOrSyntax {
		boolean loadOk = false;
		try {
			BufferedReader fr = new BufferedReader(r);
			String line = fr.readLine();
			String seqTmp = "";
			Vector<Integer> strTmp = new Vector<Integer>();

			int bpFrom;
			char base;
			int bpTo;
			int minIndex = -1;
			boolean noWarningYet = true;
			while (line != null) {
				line = line.trim();
				String[] tokens = line.split("\\s+");
				if (tokens.length == 3 && !tokens[0].contains("#")) 
				{
					try{
						base = tokens[1].charAt(tokens[1].length() - 1);

						bpFrom = (Integer.parseInt(tokens[0]));
						bpTo = (Integer.parseInt(tokens[2]));
						
						if (minIndex<0)
    						minIndex = bpFrom;
						bpFrom -= minIndex; 
						if (bpTo!=0)
						  bpTo -= minIndex;
						else
						  bpTo = -1;

						if (bpFrom != seqTmp.length()) {
							if (noWarningYet) {
								noWarningYet = false;
								warningEmition("Discontinuity detected between nucleotides "
										+ (seqTmp.length())
										+ " and "
										+ (bpFrom + 1)
										+ "!\nFilling in missing portions with unpaired unknown 'X' nucleotides ...");
							}
							while (bpFrom != seqTmp.length()) {
								seqTmp += 'X';
								strTmp.add(-1);
							}
						}
						seqTmp += base;
						strTmp.add(bpTo);
					}
					catch(Exception e5)
					{}
				}
				line = fr.readLine();
			}
			if (strTmp.size() != 0) {
				char[] seq = seqTmp.toCharArray();
				int[] str = new int[strTmp.size()];
				for (int i = 0; i < strTmp.size(); i++) {
					str[i] = strTmp.elementAt(i).intValue();
				}
				
				setRNA(seq, str, minIndex);
				loadOk = true;
			}
		}
		catch (NumberFormatException e) {
			return false;
		}
		catch (Exception e) {
			throw new ExceptionLoadingFailed(e.getMessage(), "");
		} 
		return loadOk;
	}

	private boolean loadSecStrCT(Reader r) throws ExceptionPermissionDenied,
			ExceptionLoadingFailed, ExceptionFileFormatOrSyntax {
		boolean loadOk = false;
		try {
			BufferedReader fr = new BufferedReader(r);
			String line = fr.readLine();
			String seqTmp = "";
			Vector<Integer> strTmp = new Vector<Integer>();
			int bpFrom;
			char base;
			int bpTo;
			boolean noWarningYet = true;
			int minIndex = -1;
			while (line != null) {
				line = line.trim();
				String[] tokens = line.split("\\s+");
				if (tokens.length >= 6) {
					try{
					bpFrom = (Integer.parseInt(tokens[0]));
					bpTo = (Integer.parseInt(tokens[4]));
					if (minIndex==-1)
						minIndex = bpFrom;
					bpFrom -= minIndex;
					if (bpTo!=0)
						bpTo -= minIndex;
					else
						bpTo = -1;
					base = tokens[1].charAt(tokens[1].length() - 1);
					Integer.parseInt(tokens[2]);
					Integer.parseInt(tokens[3]);
					Integer.parseInt(tokens[5]);
					if (bpFrom != seqTmp.length()) {
						if (noWarningYet) {
							noWarningYet = false;
							warningEmition("Discontinuity detected between nucleotides "
									+ (seqTmp.length())
									+ " and "
									+ (bpFrom + 1)
									+ "!\nFilling in missing portions with unpaired unknown 'X' nucleotides ...");
						}
						while (bpFrom != seqTmp.length()) {
							seqTmp += 'X';
							strTmp.add(-1);
						}
					}
					seqTmp += base;
					strTmp.add(bpTo);
					}
					catch (NumberFormatException e) {
						}
				}
				line = fr.readLine();
			}
			if (strTmp.size() != 0) {
				char[] seq = seqTmp.toCharArray();
				int[] str = new int[strTmp.size()];
				for (int i = 0; i < strTmp.size(); i++) {
					str[i] = strTmp.elementAt(i).intValue();
				}
				setRNA(seq, str, minIndex);
				loadOk = true;
			}
		} catch (IOException e) {
			throw new ExceptionLoadingFailed(e.getMessage(), "");
		} catch (NumberFormatException e) {
			throw new ExceptionFileFormatOrSyntax(e.getMessage(), "");
		}
		return loadOk;
	}

	private boolean loadSecStrRNAML(Reader r) throws ExceptionPermissionDenied,
			ExceptionLoadingFailed, ExceptionFileFormatOrSyntax {
		boolean loadOk = false;
		try {
			System.setProperty("javax.xml.parsers.SAXParserFactory", "com.sun.org.apache.xerces.internal.jaxp.SAXParserFactoryImpl");
			SAXParserFactory saxFact = javax.xml.parsers.SAXParserFactory.newInstance();
			saxFact.setValidating(false);
			saxFact.setXIncludeAware(false);
			saxFact.setNamespaceAware(false);
			SAXParser sp = saxFact.newSAXParser();
			RNAMLParser RNAMLData = new RNAMLParser();
			sp.parse(new InputSource(r), RNAMLData);
			
			/*XMLReader xr = XMLReaderFactory.createXMLReader();
			RNAMLParser RNAMLData = new RNAMLParser();
			xr.setContentHandler(RNAMLData);
			xr.setErrorHandler(RNAMLData);
			xr.setEntityResolver(RNAMLData);
			xr.parse(new InputSource(r));*/

			setRNA(RNAMLData.getSequence(), RNAMLData.getBasicPlanarStructure());
			Vector<RNAMLParser.BPTemp> bps = RNAMLData.getPlanarBPs();
			for (int i = 0; i < bps.size(); i++) {
				RNAMLParser.BPTemp bp = bps.get(i);
				int bp5 = bp.pos5 - 1;
				int bp3 = bp.pos3 - 1;
				ModeleBase mb = _listeBases.get(bp5);
				ModeleBase part = _listeBases.get(bp3);
				ModeleStyleBP newStyle = bp.createBPStyle(mb, part);
				mb.set_styleBP(newStyle);
				part.set_styleBP(newStyle);
			}

			Vector<Integer> basenumbers = RNAMLData.getBaseNumbers();
			for (int i = 0; i < _listeBases.size(); i++) {
				ModeleBaseNucleotide n = (ModeleBaseNucleotide) _listeBases
						.get(i);
				n.setBaseNumber(basenumbers.get(i));
			}

			Vector<RNAMLParser.BPTemp> bpsAux = RNAMLData.getAuxBPs();
			for (int i = 0; i < bpsAux.size(); i++) {
				RNAMLParser.BPTemp bp = bpsAux.get(i);
				int bp5 = bp.pos5 - 1;
				int bp3 = bp.pos3 - 1;
				addBPAux(bp5,bp3);
				//ModeleBase mb = _listeBases.get(bp5);
				//ModeleBase part = _listeBases.get(bp3);
				//ModeleStyleBP newStyle = bp.createBPStyle(mb, part);
				//_structureAux.add(newStyle);
			}
			loadOk = true;

		} catch (IOException ioe) {
			throw new ExceptionLoadingFailed(
					"Couldn't load file due to I/O or security policy issues.",
					"");
		} catch (Exception ge) {
          System.out.println(ge.toString());
		}
		return loadOk;
	}

	private boolean loadSecStrDBN(Reader r) throws ExceptionLoadingFailed,
			ExceptionPermissionDenied, ExceptionUnmatchedClosingParentheses,
			ExceptionFileFormatOrSyntax {
		boolean loadOk = false;
		try {
			BufferedReader fr = new BufferedReader(r);
			String line = fr.readLine();
			String title = "";
			String seqTmp = "";
			String strTmp = "";
			while ((line != null) && (strTmp.equals(""))) {
				line = line.trim();
				if (!line.startsWith(">")) {
					if (seqTmp.equals("")) {
						seqTmp = line;
					} else {
						strTmp = line;
					}
				}
				else
				{
					title = line.substring(1).trim();
				}
				line = fr.readLine();
			}
			if (strTmp.length() != 0) {
				setRNA(seqTmp, strTmp);
				this.setName(title);
				loadOk = true;
			}
		} catch (IOException e) {
			throw new ExceptionLoadingFailed(e.getMessage(), "");
		}
		return loadOk;
	}

	public void loadSecStr(Reader r) throws ExceptionFileFormatOrSyntax {
		loadSecStr(r,FILE_TYPE_UNKNOWN);
		}

	public void loadSecStr(Reader r, int fileType) throws ExceptionFileFormatOrSyntax 
	{			
		switch(fileType)
		{		
			case (FILE_TYPE_DBN):
			{
				try {
					boolean ok = loadSecStrDBN(r);
					if (ok) return;
				} catch (Exception e) { }
			}
			break;
			case (FILE_TYPE_CT):
			{
				try {
					boolean ok = loadSecStrCT(r);
					if (ok) return;
				} catch (Exception e) { }
			}
			break;
			case (FILE_TYPE_BPSEQ):
			{
				try {
					boolean ok = loadSecStrBPSEQ(r);
					if (ok) return;
				} catch (Exception e) { }
			}
			break;
			case (FILE_TYPE_RNAML):
			{
				try {
					boolean ok = loadSecStrRNAML(r);
					if (ok) return;
				} catch (Exception e) { }
			}
			break;
			case (FILE_TYPE_UNKNOWN):
			{
				BufferedReader buf = new BufferedReader(r);

				try {
					buf.mark(1000000);
					try {
						boolean ok = loadSecStrCT(buf);
						if (ok)
							return;
					} catch (Exception e) {
					}
					buf.reset();
					try {
						boolean ok = loadSecStrBPSEQ(buf);
						if (ok)
							return;
					} catch (Exception e) {
					}
					buf.reset();
					try {
						boolean ok = loadSecStrDBN(buf);
						if (ok)
							return;
					} catch (Exception e) {
						e.printStackTrace();
					}
					buf.reset();
					try {
						boolean ok = loadSecStrRNAML(buf);
						if (ok)
							return;
					} catch (ExceptionLoadingFailed e2)
					{
						e2.printStackTrace();
					} catch (Exception e) {
						e.printStackTrace();
					}
					try {
						boolean ok = loadSecStrRNAML(buf);
						if (ok)
							return;
					} catch (ExceptionLoadingFailed e2)
					{
						e2.printStackTrace();
					} catch (Exception e) {
						e.printStackTrace();
					}
					buf.reset();
				} catch (IOException e2) {
					e2.printStackTrace();
				}
			}
		}		
		throw new ExceptionFileFormatOrSyntax("");
	}

	public static int guessFileTypeFromExtension(String path)
	{
		if (path.toLowerCase().endsWith("ml"))
		{ return RNA.FILE_TYPE_RNAML; }
		else if (path.toLowerCase().endsWith("dbn")||path.toLowerCase().endsWith("faa"))
		{ return RNA.FILE_TYPE_DBN; }
		else if (path.toLowerCase().endsWith("ct"))
		{ return RNA.FILE_TYPE_CT; }
		else if (path.toLowerCase().endsWith("bpseq"))
		{ return RNA.FILE_TYPE_BPSEQ; }
		
		return RNA.FILE_TYPE_UNKNOWN; 			

	}
	
	public void loadSecStr(String path) throws ExceptionExportFailed,
			ExceptionPermissionDenied, ExceptionLoadingFailed,
			ExceptionFileFormatOrSyntax, ExceptionUnmatchedClosingParentheses,
			FileNotFoundException {
		FileReader fr = null;
		try {
			fr = new FileReader(path); 
			loadSecStr(fr,guessFileTypeFromExtension(path));
		} catch (ExceptionFileFormatOrSyntax e) {
			if (fr != null)
				try {fr.close();} catch(IOException e2){}
			e.setPath(path);
			throw e;
		}
	}

	public void set_listeBases(ArrayList<ModeleBase> _liste) {
		this._listeBases = _liste;
	}

	public void addVARNAListener(InterfaceVARNAListener rl) {
		_listeVARNAListener.add(rl);
	}

	public void warningEmition(String warningMessage) {
		for (int i = 0; i < _listeVARNAListener.size(); i++) {
			_listeVARNAListener.get(i).warningEmited(warningMessage);
		}
	}

	public void applyStyleOnBases(ArrayList<Integer> basesList,
			ModeleStyleBase style) {
		for (int i = 1; i < basesList.size(); i++) {
			_listeBases.get(basesList.get(i)).set_styleBase(style);
		}
	}

	private int[] correctReciprocity(int[] str) {
		int[] result = new int[str.length];
		for (int i = 0; i < str.length; i++) {
			if (str[i] != -1) {
				if (i == str[str[i]]) {
					result[i] = str[i];
				}
				else {
					str[str[i]] = i;
				}
			}
			else
			{
				result[i] = -1;
			}
		}
		return result;
	}

	private void applyStruct(int[] str) throws ExceptionFileFormatOrSyntax {
		str = correctReciprocity(str);

		int[] planarSubset = RNAMLParser.planarize(str);
		_structureAux.clear();

		for (int i = 0; i < planarSubset.length; i++) {
			_listeBases.get(i).set_elementStructure(planarSubset[i]);
			if (planarSubset[i] > i) {
                addBP(i,planarSubset[i]);
			}
			else if ((planarSubset[i]!=str[i]))
			{
				addBPAux(i,str[i]);
			}
		}

	}

	@SuppressWarnings("unused")
	private void initStruct(int size) {
		for (int i = 0; i < size; i++) {
			_listeBases.get(i).set_elementStructure(-1);
		}
	}

	public ArrayList<ModeleBase> get_listeBases() {
		return _listeBases;
	}

	public boolean is_comparisonMode() {
		return _comparisonMode;
	}

	public void set_comparisonMode(boolean mode) {
		_comparisonMode = mode;
	}

	public int getSize() {
		return _listeBases.size();
	}

	public ArrayList<Integer> findAll() {
		ArrayList<Integer> listAll = new ArrayList<Integer>();
		for (int i = 0; i < get_listeBases().size(); i++) {
			listAll.add(i);
		}
		return listAll;
	}

	public ArrayList<Integer> findBulge(int index) {
		ArrayList<Integer> listUp = new ArrayList<Integer>();
		if (get_listeBases().get(index).get_elementStructure() == -1) {
			int i = index;
			boolean over = false;
			while ((i < get_listeBases().size()) && !over) {
				int j = get_listeBases().get(i).get_elementStructure();
				if (j == -1) {
					listUp.add(i);
					i++;
				} else {
					over = true;
				}
			}
			i = index - 1;
			over = false;
			while ((i >= 0) && !over) {
				int j = get_listeBases().get(i).get_elementStructure();
				if (j == -1) {
					listUp.add(i);
					i--;
				} else {
					over = true;
				}
			}
		}
		return listUp;
	}

	public ArrayList<Integer> findStem(int index) {
		ArrayList<Integer> listUp = new ArrayList<Integer>();
		int i = index;
		do {
			listUp.add(i);
			int j = get_listeBases().get(i).get_elementStructure();
			if (j == -1) {
				i = (i + 1) % getSize();
			} else {
				if ((j < i) && (index <= i) && (j <= index)) {
					i = j;
				} else {
					i = (i + 1) % getSize();
				}
			}
		} while (i != index);
		return listUp;
	}

	public int getHelixCountOnLoop(int indice) {
		int cptHelice = 0;
		if (indice < 0 || indice >= get_listeBases().size())
			return cptHelice;
		int i = indice;
		int j = get_listeBases().get(i).get_elementStructure();
		// Only way to distinguish "supporting base-pair" from others
		boolean justJumped = false;
		if ((j != -1) && (j < i)) {
			i = j + 1;
			indice = i;
		}
		do {
			j = get_listeBases().get(i).get_elementStructure();
			if ((j != -1) && (!justJumped)) {
				i = j;
				justJumped = true;
				cptHelice++;
			} else {
				i = (i + 1) % get_listeBases().size();
				justJumped = false;
			}
		} while (i != indice);
		return cptHelice;
	}

	public ArrayList<Integer> findLoop(int indice) {
		return findLoopForward(indice);
	}

	public ArrayList<Integer> findLoopForward(int indice) {
		ArrayList<Integer> base = new ArrayList<Integer>();
		if (indice < 0 || indice >= get_listeBases().size())
			return base;
		int i = indice;
		int j = get_listeBases().get(i).get_elementStructure();
		// Only way to distinguish "supporting base-pair" from others
		boolean justJumped = false;
		if (j != -1) {
			i = Math.min(i, j) + 1;
			indice = i;
		}
		do {
			base.add(i);
			j = get_listeBases().get(i).get_elementStructure();
			if ((j != -1) && (!justJumped)) {
				i = j;
				justJumped = true;
			} else {
				i = (i + 1) % get_listeBases().size();
				justJumped = false;
			}
		} while (i != indice);
		return base;
	}

	public ArrayList<Integer> findPair(int indice) {
		ArrayList<Integer> base = new ArrayList<Integer>();
		int j = get_listeBases().get(indice).get_elementStructure();
		if (j != -1) {
			base.add(Math.min(indice, j));
			base.add(Math.max(indice, j));
		}

		return base;

	}

	public ArrayList<Integer> findLoopBackward(int indice) {
		ArrayList<Integer> base = new ArrayList<Integer>();
		if (indice < 0 || indice >= get_listeBases().size())
			return base;
		int i = indice;
		int j = get_listeBases().get(i).get_elementStructure();
		// Only way to distinguish "supporting base-pair" from others
		boolean justJumped = false;
		if (j != -1) {
			i = Math.min(i, j) - 1;
			indice = i;
		}
		if (i < 0) {
			return base;
		}
		do {
			base.add(i);
			j = get_listeBases().get(i).get_elementStructure();
			if ((j != -1) && (!justJumped)) {
				i = j;
				justJumped = true;
			} else {
				i = (i + get_listeBases().size() - 1) % get_listeBases().size();
				justJumped = false;
			}
		} while (i != indice);
		return base;
	}

	public ArrayList<Integer> findHelix(int indice) {
		ArrayList<Integer> list = new ArrayList<Integer>();
		if (get_listeBases().get(indice).get_elementStructure() != -1) {
			list.add(indice);
			list.add(get_listeBases().get(indice).get_elementStructure());
			int i = 1, prec = get_listeBases().get(indice)
					.get_elementStructure();
			while (indice + i < get_listeBases().size()
					&& get_listeBases().get(indice + i).get_elementStructure() != -1
					&& get_listeBases().get(indice + i).get_elementStructure() == prec - 1) {
				list.add(indice + i);
				list.add(get_listeBases().get(indice + i)
						.get_elementStructure());
				prec = get_listeBases().get(indice + i).get_elementStructure();
				i++;
			}
			i = -1;
			prec = get_listeBases().get(indice).get_elementStructure();
			while (indice + i >= 0
					&& get_listeBases().get(indice + i).get_elementStructure() != -1
					&& get_listeBases().get(indice + i).get_elementStructure() == prec + 1) {
				list.add(indice + i);
				list.add(get_listeBases().get(indice + i)
						.get_elementStructure());
				prec = get_listeBases().get(indice + i).get_elementStructure();
				i--;
			}
		}
		return list;
	}

	public ArrayList<Integer> find3Prime(int indice) {
		ArrayList<Integer> list = new ArrayList<Integer>();
		boolean over = false;
		while ((indice >= 0) && !over) {
			over = (get_listeBases().get(indice).get_elementStructure() != -1);
			indice--;
		}
		indice++;
		if (over) {
			indice++;
		}
		for (int i = indice; i < get_listeBases().size(); i++) {
			list.add(i);
			if (get_listeBases().get(i).get_elementStructure() != -1) {
				return new ArrayList<Integer>();
			}
		}
		return list;
	}

	public ArrayList<Integer> find5Prime(int indice) {
		ArrayList<Integer> list = new ArrayList<Integer>();
		for (int i = 0; i <= indice; i++) {
			list.add(i);
			if (get_listeBases().get(i).get_elementStructure() != -1) {
				return new ArrayList<Integer>();
			}
		}
		return list;
	}

	public Double get_spaceBetweenBases() {
		return _spaceBetweenBases;
	}

	public void set_spaceBetweenBases(Double betweenBases) {
		_spaceBetweenBases = betweenBases;
	}

	public static Double angle(Point2D.Double p1, Point2D.Double p2,
			Point2D.Double p3) {
		Double alpha = Math.atan2(p1.y - p2.y, p1.x - p2.x);
		Double beta = Math.atan2(p3.y - p2.y, p3.x - p2.x);
		Double angle = (beta - alpha);

		// Correction de l'angle pour le resituer entre 0 et 2PI
		while (angle < 0.0 || angle > 2 * Math.PI) {
			if (angle < 0.0)
				angle += 2 * Math.PI;
			else if (angle > 2 * Math.PI)
				angle -= 2 * Math.PI;
		}
		return angle;
	}

	public ArrayList<Integer> findNonPairedBaseGroup(Integer get_nearestBase) {
		// detection 3', 5', bulge
		ArrayList<Integer> list = new ArrayList<Integer>();
		int indice = get_nearestBase;
		boolean nonpairedUp = true, nonpairedDown = true;
		while (indice < get_listeBases().size() && nonpairedUp) {
			if (get_listeBases().get(indice).get_elementStructure() == -1) {
				list.add(indice);
				indice++;
			} else {
				nonpairedUp = false;
			}
		}
		indice = get_nearestBase - 1;
		while (indice >= 0 && nonpairedDown) {
			if (get_listeBases().get(indice).get_elementStructure() == -1) {
				list.add(indice);
				indice--;
			} else {
				nonpairedDown = false;
			}
		}
		return list;
	}

	public boolean getDrawn() {
		return _drawn;
	}

	public ArrayList<ModeleStyleBP> getStructureAux() {
		return _structureAux;
	}

	public int getIndexFromBaseNumber(int num) {
		for (int i = 0; i < this._listeBases.size(); i++) {
			if (_listeBases.get(i).getBaseNumber() == num) {
				return i;
			}
		}
		return -1;
	}

	/**
	 * Adds a base pair to this RNA's structure. Tries to add it to the
	 * secondary structure first, eventually adding it to the 'tertiary'
	 * interactions if it clashes with the current secondary structure.
	 * 
	 * @param a
	 *            - Base number of the origin of this base pair
	 * @param b
	 *            - Base number of the destination of this base pair
	 */

	public void addBPToStructure(int a, int b) {
		int i = getIndexFromBaseNumber(a);
		int j = getIndexFromBaseNumber(b);
		ModeleBase part5 = _listeBases.get(i);
		ModeleBase part3 = _listeBases.get(j);
		ModeleStyleBP msbp = new ModeleStyleBP(part5, part3);
		addBPToStructure(a, b, msbp);
	}

	/**
	 * Adds a base pair to this RNA's structure. Tries to add it to the
	 * secondary structure first, possibly adding it to the 'tertiary'
	 * interactions if it clashes with the current secondary structure.
	 * 
	 * @param a
	 *            - Base number of the origin of this base pair
	 * @param b
	 *            - Base number of the destination of this base pair
	 */

	public void addBPToStructure(int a, int b, ModeleStyleBP msbp) {
		int i = getIndexFromBaseNumber(a);
		int j = getIndexFromBaseNumber(b);
		if (j < i) {
			int k = j;
			j = i;
			i = k;
		}
		if (i != -1) {
			if ((_listeBases.get(i).get_elementStructure() != -1)
					|| (_listeBases.get(j).get_elementStructure() != -1)) {
				addBPAux(i, j, msbp);
				return;
			}

			for (int k = i + 1; k < j; k++) {
				ModeleBase tmp = _listeBases.get(k);
				int l = tmp.get_elementStructure();
				if (l != -1) {
					if ((l <= i) || (l >= j)) {
						addBPAux(i, j, msbp);
						return;
					}
				}
			}
			addBP(i, j, msbp);
		}
	}

	public void addBP(int i, int j) {
		if (j < i) {
			int k = j;
			j = i;
			i = k;
		}

		ModeleBase part5 = _listeBases.get(i);
		ModeleBase part3 = _listeBases.get(j);
		ModeleStyleBP msbp = new ModeleStyleBP(part5, part3);
		addBP(i, j, msbp);
	}

	public void addBP(int i, int j, ModeleStyleBP msbp) {
		if (j < i) {
			int k = j;
			j = i;
			i = k;
		}
		ModeleBase part5 = _listeBases.get(i);
		ModeleBase part3 = _listeBases.get(j);
		msbp.setPartner5(part5);
		msbp.setPartner3(part3);
		part5.set_elementStructure(j, msbp);
		part3.set_elementStructure(i, msbp);
	}

	public void addBPAux(int i, int j) {
		ModeleBase part5 = _listeBases.get(i);
		ModeleBase part3 = _listeBases.get(j);
		ModeleStyleBP msbp = new ModeleStyleBP(part5, part3);
		addBPAux(i, j, msbp);
	}

	public void addBPAux(int i, int j, ModeleStyleBP msbp) {
		if (j < i) {
			int k = j;
			j = i;
			i = k;
		}
		ModeleBase part5 = _listeBases.get(i);
		ModeleBase part3 = _listeBases.get(j);
		msbp.setPartner5(part5);
		msbp.setPartner3(part3);
		_structureAux.add(msbp);
	}

	public ModeleStyleBP getBPStyle(int i, int j) {
		ModeleStyleBP result = null;
		if (i > j) {
			int k = j;
			j = i;
			i = k;
		}
		if (_listeBases.get(i).get_elementStructure() == j) {
			result = _listeBases.get(i).get_styleBP();
		}
		for (int k = 0; k < _structureAux.size(); k++) {
			ModeleStyleBP bp = _structureAux.get(k);
			if ((bp.getPartner5().getIndex() == i)
					&& (bp.getPartner3().getIndex() == j)) {
				result = bp;
			}
		}
		return result;
	}

	public void setBaseInnerColor(Color c) {
		for (int i = 0; i < _listeBases.size(); i++) {
			ModeleBase mb = _listeBases.get(i);
			mb.get_styleBase().set_base_inner_color(c);
		}
	}

	public void setBaseNumbersColor(Color c) {
		for (int i = 0; i < _listeBases.size(); i++) {
			ModeleBase mb = _listeBases.get(i);
			mb.get_styleBase().set_base_number_color(c);
		}
	}

	public void setBaseNameColor(Color c) {
		for (int i = 0; i < _listeBases.size(); i++) {
			ModeleBase mb = _listeBases.get(i);
			mb.get_styleBase().set_base_name_color(c);
		}
	}

	public void setBaseOutlineColor(Color c) {
		for (int i = 0; i < _listeBases.size(); i++) {
			ModeleBase mb = _listeBases.get(i);
			mb.get_styleBase().set_base_outline_color(c);
		}
	}

	public boolean getFlatExteriorLoop()
	{
		return _flatExteriorLoop;
	}

	public void setFlatExteriorLoop(boolean on)
	{
		_flatExteriorLoop = on;
	}
	
	public String getName()
	{
		return _name;
	}

	public void setName(String n)
	{
		_name = n;
	}
	
	public ArrayList<TextAnnotation> getAnnotations()
	{
		return _listeAnnotations;
	}

	public boolean removeAnnotation(TextAnnotation t)
	{
		return _listeAnnotations.remove(t);
	}

	public void addAnnotation(TextAnnotation t)
	{
		_listeAnnotations.add(t);
	}

	public void clearAnnotations()
	{
		_listeAnnotations.clear();
	}
	
	public void autoAnnotateHelices()
	{
		Stack<Integer> p = new Stack<Integer>();
		p.push(0);
		int nbH = 1;
		while(!p.empty())
		{
			int i = p.pop();
			if (i<_listeBases.size())
			{
				ModeleBase mb = _listeBases.get(i); 
				int j = mb.get_elementStructure(); 
				if (j==-1)
				{ p.push(i+1);	}
				else
				{
					if (j>i)
					{
						ModeleBase mbp = _listeBases.get(j); 
						p.push(j+1);
						ArrayList<ModeleBase> h = new ArrayList<ModeleBase>(); 
						int k = 1;
						while(mb.get_elementStructure()==mbp.getIndex())
						{
							h.add(mb);
							h.add(mbp);
							mb = _listeBases.get(i+k);
							mbp = _listeBases.get(j-k); 
						
							k++;
						}
						try {
							addAnnotation(new TextAnnotation("H"+nbH++, h , TextAnnotation.HELIX));
						} catch (Exception e) {
							e.printStackTrace();
						}
						p.push(i+k);
					}
				}
			}
		}
	}

	public void autoAnnotateTerminalLoops()
	{
		Stack<Integer> p = new Stack<Integer>();
		p.push(0);
		int nbT = 1;
		while(!p.empty())
		{
			int i = p.pop();
			if (i<_listeBases.size())
			{
			ModeleBase mb = _listeBases.get(i); 
			int j = mb.get_elementStructure(); 
			if (j==-1)
			{ 	
				int k = 1;
				ArrayList<ModeleBase> t = new ArrayList<ModeleBase>();
				while ((i+k<getSize())&&(mb.get_elementStructure()==-1))
				{
					t.add(mb);
					mb = _listeBases.get(i+k);
					k++;
				}
				if (mb.get_elementStructure()!=-1)
				{
					if (mb.get_elementStructure()==i-1)
					{
						try {
							t.add(_listeBases.get(i-1));
							t.add(_listeBases.get(i+k-1));
							addAnnotation(new TextAnnotation("T"+nbT++, t , TextAnnotation.LOOP));
						} catch (Exception e) {
							e.printStackTrace();
						}
					}
					p.push(i+k-1);
				}
				
			}
			else
			{
				if (j>i)
				{
					p.push(j+1);
					p.push(i+1);
				}
			}
			}
		}
	}

	public void autoAnnotateInteriorLoops()
	{
		Stack<Integer> p = new Stack<Integer>();
		p.push(0);
		int nbT = 1;
		while(!p.empty())
		{
			int i = p.pop();
			if (i<_listeBases.size())
			{
				ModeleBase mb = _listeBases.get(i); 
				int j = mb.get_elementStructure(); 
				if (j==-1)
				{ 	
					int k = i+1;
					ArrayList<ModeleBase> t = new ArrayList<ModeleBase>();
					boolean terminal = true;
					while ((k<getSize())&&((mb.get_elementStructure()>=i)||(mb.get_elementStructure()==-1)))
					{
						t.add(mb);
						mb = _listeBases.get(k);
						if ((mb.get_elementStructure()==-1)||(mb.get_elementStructure()<k))
							k++;
						else 
						{
							p.push(k);
							terminal = false;
							k = mb.get_elementStructure();
						}
					}
					if (mb.get_elementStructure()!=-1)
					{
						if ((mb.get_elementStructure()==i-1)&& !terminal)
						{
							try {
								t.add(_listeBases.get(i-1));
								t.add(_listeBases.get(k-1));
								addAnnotation(new TextAnnotation("I"+nbT++, t , TextAnnotation.LOOP));
							} catch (Exception e) {
								e.printStackTrace();
							}
							p.push(k-1);
						}
					}	
				}
				else
				{
				if (j>i)
				{
					p.push(i+1);
				}
			}
		}
		}
	}
	
	@SuppressWarnings("unchecked")
	public TextAnnotation getAnnotation(int type, ModeleBase base)
	{
		TextAnnotation result = null;
		for(TextAnnotation t : _listeAnnotations)
		{
			if (t.getType()==type)
			{
				switch(type)
				{
				    case(TextAnnotation.BASE):
				    	if (base == (ModeleBase) t.getAncrage())
				    	  return t;
					break;
				    case(TextAnnotation.HELIX):
				    case(TextAnnotation.LOOP):
				    {
				    	ArrayList<ModeleBase> mbl = (ArrayList<ModeleBase>) t.getAncrage();
				    	if (mbl.contains(base))
				    	  return t;
				    }
					break;
				}
			}
		}
		return result;
	}
}


